@@ -630,22 +630,27 @@ domGetNodeName(xmlNodePtr node, int xpath_key) {
         break;
 
     case XML_CDATA_SECTION_NODE :
-        name = (const xmlChar*) (xpath_key ? "text()" : "#cdata-section");
+        name = (const xmlChar*) (xpath_key ? "text()" : "#cdata");
         break;
 
     case XML_TEXT_NODE :
         name = (const xmlChar*) (xpath_key ? "text()" : "#text");
         break;
 
-
     case XML_DOCUMENT_NODE :
+        name = (const xmlChar*) (xpath_key ? "document()" : "#xml");
+        break;
+
     case XML_HTML_DOCUMENT_NODE :
+        name = (const xmlChar*) (xpath_key ? "document()" : "#html");
+        break;
+
     case XML_DOCB_DOCUMENT_NODE :
-        name = (const xmlChar*) (xpath_key ? "document()" : "#document");
+        name = (const xmlChar*) (xpath_key ? "document()" : "#docbook");
         break;
 
     case XML_DOCUMENT_FRAG_NODE :
-        name = (const xmlChar*) (xpath_key ? "document()" : "#document-fragment");
+        name = (const xmlChar*) (xpath_key ? "document()" : "#fragment");
         break;
 
     case XML_PI_NODE :
@@ -692,14 +697,22 @@ domGetNodeName(xmlNodePtr node, int xpath_key) {
                 }
             }
         }
-        if (node->type == XML_ATTRIBUTE_NODE) {
+        else if (node->type == XML_ATTRIBUTE_NODE) {
             // prepend '@'
-            xmlChar* at_key = xmlStrdup( (xmlChar*) "@" );
-            at_key = xmlStrcat(at_key, rv);
+            xmlChar* name = xmlStrdup( (xmlChar*) "@" );
+            name = xmlStrcat(name, rv);
             xmlFree(rv);
-            rv = at_key;
+            rv = name;
         }
-    } 
+    }
+    else if (node->type == XML_PI_NODE) {
+            // prepend '?' to processing instructions
+            xmlChar* name = xmlStrdup( (xmlChar*) "?" );
+            name = xmlStrcat(name, rv);
+            xmlFree(rv);
+            rv = name;
+        }
+
 
    return rv;
 }
@@ -712,6 +725,11 @@ domSetNodeName(xmlNodePtr self , xmlChar *string) {
     if (self == NULL || string == NULL || *string == 0)
         return;
 
+    if (self->type == XML_PI_NODE && *string == '?') {
+        // skip leading '?'
+        string++;
+    }
+
     if( ( self->type == XML_ELEMENT_NODE
           || self->type == XML_ATTRIBUTE_NODE
           || self->type == XML_PI_NODE)
@@ -753,6 +771,15 @@ domAppendChild( xmlNodePtr self,
         return newChild;
     }
 
+    if (self->type == XML_ELEMENT_NODE) {
+        if (newChild->type == XML_ATTRIBUTE_NODE) {
+            return (xmlNodePtr)domSetAttributeNodeNS(self, (xmlAttrPtr) newChild );
+        }
+        if (newChild->type == XML_NAMESPACE_DECL) {
+            xmlNsPtr ns = (xmlNsPtr) newChild;
+            return (xmlNodePtr) xmlNewNs(self, ns->href, ns->prefix);
+        }
+    }
     if ( !(domTestHierarchy(self, newChild)
            && domTestDocument(self, newChild))){
         xml6_fail(self, "appendChild: HIERARCHY_REQUEST_ERR");
@@ -1125,35 +1152,89 @@ domSetNodeValue( xmlNodePtr n , xmlChar* val ){
     }
 }
 
+DLLEXPORT int
+domNodeType(xmlChar* name) {
+    int node_type = 0;
+
+    if (name != NULL) {
+        switch (*name) {
+            case '?' :
+                node_type = XML_PI_NODE;
+                break;
+            case '#': {
+                if (xmlStrcmp( name, (unsigned char*) "#xml" ) == 0) {
+                    node_type = XML_DOCUMENT_NODE;
+                }
+                else if (xmlStrcmp( name, (unsigned char*) "#html" ) == 0) {
+                    node_type = XML_HTML_DOCUMENT_NODE;
+                }
+                else if (xmlStrcmp( name, (unsigned char*) "#docbook" ) == 0) {
+                    node_type = XML_DOCUMENT_NODE;
+                }
+                else if (xmlStrcmp( name, (unsigned char*) "#fragment" ) == 0) {
+                    node_type = XML_DOCUMENT_FRAG_NODE;
+                }
+                else if (xmlStrcmp( name, (unsigned char*) "#text" ) == 0) {
+                    node_type = XML_TEXT_NODE;
+                }
+                else if (xmlStrcmp( name, (unsigned char*) "#comment" ) == 0) {
+                    node_type = XML_COMMENT_NODE;
+                }
+                else if (xmlStrcmp( name, (unsigned char*) "#cdata" ) == 0) {
+                    node_type = XML_CDATA_SECTION_NODE;
+                }
+                else {
+                    fprintf(stderr, __FILE__ "%d: unknown node generic name '%s'\n", __LINE__, name);
+                }
+                break;
+            }
+            default: {
+                node_type = XML_ELEMENT_NODE;
+                break;
+            }
+        }
+    }
+
+    return node_type;
+}
+
 DLLEXPORT xmlNodeSetPtr
 domGetChildrenByLocalName( xmlNodePtr self, xmlChar* name ){
     xmlNodeSetPtr rv = NULL;
-    xmlNodePtr cld = NULL;
+    xmlNodePtr cld = self->children;
     int node_type = 0;
 
     if ( self != NULL && name != NULL ) {
-        if (xmlStrcmp( name, (unsigned char*) "*" ) == 0) {
-            node_type = XML_ELEMENT_NODE;
-        }
-        else if (*name == '#') {
-            // See domGetNodeName()
-            if (xmlStrcmp( name, (unsigned char*) "#text" ) == 0) {
-                node_type = XML_TEXT_NODE;
+
+        switch (*name) {
+            case '*' : {     // -- Element wildcard
+                name = NULL;
+                node_type = XML_ELEMENT_NODE;
+                break;
             }
-            else if (xmlStrcmp( name, (unsigned char*) "#comment" ) == 0) {
-                node_type = XML_COMMENT_NODE;
+            case '?' : {     // -- Named PI node
+                name++; // skip leading '?'
+                if (*name == '*') name = NULL; // "?*" wildcard
+                node_type = XML_PI_NODE;
+                break;
             }
-            else if (xmlStrcmp( name, (unsigned char*) "#cdata-section" ) == 0) {
-                node_type = XML_CDATA_SECTION_NODE;
+            case '@' : {     // -- Named Attribute
+                name++; // skip leading '@'
+                if (*name == '*') name = NULL; // "@*" wildcard
+                node_type = XML_ATTRIBUTE_NODE;
+                cld = (xmlNodePtr) self->properties; // scan attributes instead of children
+                break;
             }
-            else {
-                fprintf(stderr, __FILE__ "%d: unable to select nodes with local-name '%s'\n", __LINE__, name);
+            default : {      // anything else
+                node_type = domNodeType(name);
+                if (*name == '#') name = NULL; // generic name
+                break;
             }
         }
-        cld = self->children;
+
         while ( cld != NULL ) {
-            if ( ((node_type && cld->type == node_type)
-                  || xmlStrcmp( name, cld->name ) == 0 )) {
+            if ( cld->type == node_type
+              && (name == NULL || xmlStrcmp( name, cld->name ) == 0 )) {
                 if ( rv == NULL ) {
                     rv = xmlXPathNodeSetCreate( cld ) ;
                 }
@@ -1240,26 +1321,6 @@ domGetChildrenByTagNameNS( xmlNodePtr self, xmlChar* nsURI, xmlChar* name ){
     return rv;
 }
 
-DLLEXPORT xmlNsPtr
-domNewNs ( xmlNodePtr elem , xmlChar* prefix, xmlChar* href ) {
-    xmlNsPtr ns = NULL;
-
-    if (elem != NULL) {
-        ns = xmlSearchNs( elem->doc, elem, prefix );
-    }
-    /* prefix is not in use */
-    if (ns == NULL) {
-        ns = xmlNewNs( elem , href , prefix );
-    } else {
-        /* prefix is in use; if it has same URI, let it go, otherwise it's
-           an error */
-        if (!xmlStrEqual(href, ns->href)) {
-            ns = NULL;
-        }
-    }
-    return ns;
-}
-
 DLLEXPORT xmlAttrPtr
 domGetAttributeNode(xmlNodePtr node, const xmlChar* qname) {
     xmlChar*  prefix    = NULL;
